{% extends "base.html" %}

{% block site_meta %}
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1">
    {% if page.meta and page.meta.description %}
      <meta name="description" content="{{ page.meta.description }}">
    {% elif config.site_description %}
      <meta name="description" content="{{ config.site_description }}">
    {% endif %}
    {% if page.meta and page.meta.author %}
      <meta name="author" content="{{ page.meta.author }}">
    {% elif config.site_author %}
      <meta name="author" content="{{ config.site_author }}">
    {% endif %}
    {% if page.canonical_url %}
      <link rel="canonical" href="{{ page.canonical_url }}">
    {% endif %}
    {% if page.previous_page %}
      <link rel="prev" href="{{ page.previous_page.url | url }}">
    {% endif %}
    {% if page.next_page %}
      <link rel="next" href="{{ page.next_page.url | url }}">
    {% endif %}
    {% if "rss" in config.plugins %}
      <link rel="alternate" type="application/rss+xml" title="{{ lang.t('rss.created') }}" href="{{ 'feed_rss_created.xml' | url }}">
      <link rel="alternate" type="application/rss+xml" title="{{ lang.t('rss.updated') }}" href="{{ 'feed_rss_updated.xml' | url }}">
    {% endif %}
    <link rel="icon" href="{{ config.theme.favicon_png | url }}" sizes="any">
    <link rel="icon" href="{{ config.theme.favicon | url }}" type="image/svg+xml">
    <meta name="generator" content="mkdocs-{{ mkdocs_version }}, mkdocs-material-9.6.14">
{% endblock %}
